## R CMD check results

0 errors | 0 warnings | 0 note

## revdepcheck results

This should fixe the failures for httr2 and filearray. I've informed/helped all other maintainers about the breakage (details at https://github.com/r-lib/waldo/issues/208). We have confirmed the fix on our cloud check server so it hopefully won't happen again.
